package javax.sql.rowset.serial;

import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Arrays;
import sun.reflect.CallerSensitive;
import sun.reflect.misc.ReflectUtil;
import sun.reflect.Reflection;
import javax.sql.rowset.RowSetWarning;
import java.util.Vector;
import java.lang.reflect.Field;
import java.io.Serializable;

public class SerialJavaObject implements Serializable, Cloneable
{
    private Object obj;
    private transient Field[] fields;
    static final long serialVersionUID = -1465795139032831023L;
    Vector<RowSetWarning> chain;
    
    public SerialJavaObject(Object obj) throws SerialException {
        Class<Object> class1 = obj.getClass();
        if (!(obj instanceof Serializable)) {
            this.setWarning(new RowSetWarning("Warning, the object passed to the constructor does not implement Serializable"));
        }
        this.fields = class1.getFields();
        if (hasStaticFields(this.fields)) {
            throw new SerialException("Located static fields in object instance. Cannot serialize");
        }
        this.obj = obj;
        return;
    }
    
    public Object getObject() throws SerialException {
        return this.obj;
    }
    
    @CallerSensitive
    public Field[] getFields() throws SerialException {
        if (this.fields != null) {
            Class<Object> class1 = this.obj.getClass();
            if (System.getSecurityManager() != null && ReflectUtil.needsPackageAccessCheck(Reflection.getCallerClass().getClassLoader(), class1.getClassLoader())) {
                ReflectUtil.checkPackageAccess(class1);
            }
            return class1.getFields();
        }
        else {
            throw new SerialException("SerialJavaObject does not contain a serialized object instance");
        }
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof SerialJavaObject && this.obj.equals(((SerialJavaObject)o).obj));
    }
    
    @Override
    public int hashCode() {
        return 31 + this.obj.hashCode();
    }
    
    public Object clone() {
        try {
            SerialJavaObject serialJavaObject = (SerialJavaObject)super.clone();
            serialJavaObject.fields = (Field[])Arrays.copyOf(this.fields, this.fields.length);
            if (this.chain != null) {
                serialJavaObject.chain = new Vector(this.chain);
                return serialJavaObject;
            }
            return serialJavaObject;
        }
        catch (CloneNotSupportedException ex) {
            throw new InternalError();
        }
    }
    
    private void setWarning(RowSetWarning e) {
        if (this.chain == null) {
            this.chain = new Vector();
        }
        this.chain.add(e);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        Vector<Object> c = (Vector)fields.get("chain", null);
        if (c != null) {
            this.chain = new Vector(c);
        }
        this.obj = fields.get("obj", null);
        if (this.obj != null) {
            this.fields = this.obj.getClass().getFields();
            if (hasStaticFields(this.fields)) {
                throw new IOException("Located static fields in object instance. Cannot serialize");
            }
            return;
        }
        else {
            throw new IOException("Object cannot be null!");
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("obj", this.obj);
        putFields.put("chain", this.chain);
        objectOutputStream.writeFields();
    }
    
    private static boolean hasStaticFields(Field[] array) {
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (array[i].getModifiers() == 8) {
                return true;
            }
            ++i;
        }
        return false;
    }
}
